@@ -65,10 +65,10 @@
 namespace Glom
 {
 
-static const int GLOM_RESPONSE_BROWSE_NETWORK = 1;
+//static const int GLOM_RESPONSE_BROWSE_NETWORK = 1;
 
 // Global application variable
-App_Glom* global_application = NULL;
+App_Glom* global_application = 0;
 
 App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : type_base(cobject, "Glom"),
@@ -98,6 +98,7 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& bu
   add_mime_type("application/x-glom"); //TODO: make this actually work - we need to register it properly.
   
 #ifndef GLOM_ENABLE_CLIENT_ONLY
+#ifndef G_OS_WIN32
   //Install UI hooks for this:
   ConnectionPool* connection_pool = ConnectionPool::get_instance();
   if(!connection_pool)
@@ -105,6 +106,7 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& bu
       sigc::mem_fun(*this, &App_Glom::on_connection_avahi_begin),
       sigc::mem_fun(*this, &App_Glom::on_connection_avahi_progress),
       sigc::mem_fun(*this, &App_Glom::on_connection_avahi_done) );     
+#endif
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   global_application = this;
@@ -355,6 +357,18 @@ void App_Glom::on_appmenu_button_table_value_changed()
 #endif //GLOM_ENABLE_MAEMO
 
 #ifdef GLOM_ENABLE_MAEMO
+static void add_button_to_appmenu(Hildon::AppMenu& appmenu, const Glib::ustring& title, const Glib::ustring& secondary, const sigc::slot<void>& clicked_handler)
+{
+  Hildon::Button* button = 
+    Gtk::manage(new Hildon::Button(
+      Gtk::Hildon::SIZE_AUTO, 
+      Hildon::BUTTON_ARRANGEMENT_VERTICAL,
+      title, secondary));
+  button->show();
+  button->signal_clicked().connect(clicked_handler);
+  appmenu.append(*button);
+}
+
 void App_Glom::init_menus()
 {
   //There is no real menu on Maemo. We use HildonAppMenu instead.
@@ -365,17 +379,14 @@ void App_Glom::init_menus()
     sigc::mem_fun(*this, &App_Glom::on_appmenu_button_table_value_changed) );
   m_maemo_appmenu.append(m_appmenu_button_table);
   
-  Hildon::Button* find_button = 
-    Gtk::manage(new Hildon::Button(
-      Gtk::Hildon::SIZE_AUTO, 
-      Hildon::BUTTON_ARRANGEMENT_VERTICAL,
-      _("Find"),
-      _("Search for records in the table.")));
-  find_button->show();
-  find_button->signal_clicked().connect(
+  add_button_to_appmenu(m_maemo_appmenu, 
+    _("Find"), _("Search for records in the table"),
     sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Mode_Find) );
-  m_maemo_appmenu.append(*find_button);
-  
+    
+  add_button_to_appmenu(m_maemo_appmenu, 
+    _("Add Record"), _("Create a new record in the table"),
+    sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_add_record) );
+
   //set_app_menu(*appmenu); //TODO: Use this instead?
   Hildon::Program::get_instance()->set_common_app_menu(m_maemo_appmenu);
 }
@@ -688,7 +699,7 @@ void App_Glom::open_browsed_document(const EpcServiceInfo* server, const Glib::u
       epc_consumer_set_username(consumer, username.c_str());
       epc_consumer_set_password(consumer, password.c_str());
 
-      GError *error = NULL;
+      GError *error = 0;
       document_contents = (gchar*)epc_consumer_lookup(consumer, "document", &length, &error);
       if(error)
       {
@@ -717,7 +728,7 @@ void App_Glom::open_browsed_document(const EpcServiceInfo* server, const Glib::u
     }
 
     delete dialog_connection;
-    dialog_connection = NULL;
+    dialog_connection = 0;
 
   }
 
@@ -1281,10 +1292,18 @@ void App_Glom::update_userlevel_ui()
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
+#ifndef GLOM_ENABLE_MAEMO
 Glib::RefPtr<Gtk::UIManager> App_Glom::get_ui_manager()
 {
   return m_refUIManager;
 }
+#else
+Hildon::AppMenu* App_Glom::get_maemo_appmenu()
+{
+  return &m_maemo_appmenu;
+}
+#endif //GLOM_ENABLE_MAEMO
+
 
 bool App_Glom::offer_new_or_existing()
 {
@@ -1530,7 +1549,7 @@ bool App_Glom::recreate_database(bool& user_cancelled)
     sharedptr<SharedConnection> sharedconnection = connection_pool->connect();
 #else
     sharedptr<SharedConnection> sharedconnection = connection_pool->connect(error);
-    if(error.get() == NULL)
+    if(!error.get())
     {
 #endif // GLIBMM_EXCEPTIONS_ENABLED
       g_warning("App_Glom::recreate_database(): Failed because database exists already.");
@@ -1544,7 +1563,7 @@ bool App_Glom::recreate_database(bool& user_cancelled)
   catch(const ExceptionConnection& ex)
   {
 #else
-  if(error.get() != NULL)
+  if(error.get())
   {
     const ExceptionConnection* exptr = dynamic_cast<ExceptionConnection*>(error.get());
     if(exptr)
@@ -1611,7 +1630,7 @@ bool App_Glom::recreate_database(bool& user_cancelled)
     sharedconnection = connection_pool->connect();
 #else
     sharedconnection = connection_pool->connect(error);
-    if(error.get() == NULL)
+    if(!error.get())
 #endif // GLIBMM_EXCEPTIONS_ENABLED
       connection_pool->set_database(db_name); //The database was successfully created, so specify it when connecting from now on.
   }
@@ -1619,7 +1638,7 @@ bool App_Glom::recreate_database(bool& user_cancelled)
   catch(const ExceptionConnection& ex)
   {
 #else
-  if(error.get() != NULL)
+  if(error.get())
   {
     const std::exception& ex = *error.get();
 #endif // GLIBMM_EXCEPTIONS_ENABLED
@@ -1807,7 +1826,7 @@ void App_Glom::fill_menu_tables()
   catch(const Glib::Error& ex)
   {
 #else
-  if(error.get() != NULL)
+  if(error.get())
   {
     const Glib::Error& ex = *error.get();
 #endif // GLIBMM_EXCEPTIONS_ENABLED
@@ -1818,7 +1837,7 @@ void App_Glom::fill_menu_tables()
 #endif //GLOM_ENABLE_MAEMO
 
 #ifdef GLOM_ENABLE_MAEMO
-void App_Glom::fill_menu_reports(const Glib::ustring& table_name)
+void App_Glom::fill_menu_reports(const Glib::ustring& /* table_name */)
 {
   //TODO: Change the Hildon::AppMenu.
 }
@@ -1906,7 +1925,7 @@ void App_Glom::fill_menu_reports(const Glib::ustring& table_name)
   catch(const Glib::Error& ex)
   {
 #else
-  if(error.get() != NULL)
+  if(error.get())
   {
     const Glib::Error& ex = *error.get();
 #endif // GLIBMM_EXCEPTIONS_ENABLED
@@ -1916,7 +1935,7 @@ void App_Glom::fill_menu_reports(const Glib::ustring& table_name)
 #endif //GLOM_ENABLE_MAEMO
 
 #ifdef GLOM_ENABLE_MAEMO
-void App_Glom::fill_menu_print_layouts(const Glib::ustring& table_name)
+void App_Glom::fill_menu_print_layouts(const Glib::ustring& /* table_name */)
 {
   //TODO: Change the Hildon::AppMenu.
 }
@@ -2018,7 +2037,7 @@ void App_Glom::fill_menu_print_layouts(const Glib::ustring& table_name)
   catch(const Glib::Error& ex)
   {
 #else
-  if(error.get() != NULL)
+  if(error.get())
   {
     const Glib::Error& ex = *error.get();
 #endif // GLIBMM_EXCEPTIONS_ENABLED
