@@ -18,150 +18,30 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libglom/libglom_config.h> // For GLOM_ENABLE_MAEMO
+#include <libglom/libglom_config.h>
  
 #include <libglom/connectionpool.h>
-#include <libglom/document/document_glom.h>
-#include <bakery/bakery.h>
+#include <libglom/document/document.h>
+#include <libglom/utils.h>
 //#include <libgdamm/connectionevent.h>
 #include <glibmm/i18n.h>
 
-#ifndef G_OS_WIN32
+#ifdef G_OS_WIN32
+# include <windows.h>
+#else
 # include <libepc/shell.h> //For epc_shell_set_progress_hooks().
 # include <libepc/publisher.h>
-#else
-// objidl.h, included by windows.h, defines a type called DATADIR, so we need
-// to undef it temporarily.
-# define GLOM_SAVE_DATADIR DATADIR
-# undef DATADIR
-# include <windows.h>
-# define DATADIR GLOM_SAVE_DATADIR
-#endif
-
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/note.h>
 #endif
 
 #include <signal.h> //To catch segfaults
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifndef G_OS_WIN32
 static EpcProtocol publish_protocol = EPC_PROTOCOL_HTTPS;
 #endif
-#endif
 
 // Uncomment to see debug messages
 //#define GLOM_CONNECTION_DEBUG
 
-namespace
-{
-
-#ifdef G_OS_WIN32
-static BOOL
-pgwin32_get_dynamic_tokeninfo(HANDLE token, TOKEN_INFORMATION_CLASS class_,
-                char **InfoBuffer, char *errbuf, int errsize)
-{
-  DWORD    InfoBufferSize;
-
-  if(GetTokenInformation(token, class_, NULL, 0, &InfoBufferSize))
-  {
-    snprintf(errbuf, errsize, "could not get token information: got zero size\n");
-    return FALSE;
-  }
-
-  if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
-  {
-    snprintf(errbuf, errsize, "could not get token information: error code %d\n",
-         (int) GetLastError());
-    return FALSE;
-  }
-
-  *InfoBuffer = static_cast<char*>(malloc(InfoBufferSize));
-  if(*InfoBuffer == NULL)
-  {
-    snprintf(errbuf, errsize, "could not allocate %d bytes for token information\n",
-         (int) InfoBufferSize);
-    return FALSE;
-  }
-
-  if(!GetTokenInformation(token, class_, *InfoBuffer,
-               InfoBufferSize, &InfoBufferSize))
-  {
-    snprintf(errbuf, errsize, "could not get token information: error code %d\n",
-         (int) GetLastError());
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-int
-pgwin32_is_admin(void)
-{
-  HANDLE    AccessToken;
-  char     *InfoBuffer = NULL;
-  char    errbuf[256];
-  PTOKEN_GROUPS Groups;
-  PSID    AdministratorsSid;
-  PSID    PowerUsersSid;
-  SID_IDENTIFIER_AUTHORITY NtAuthority = {SECURITY_NT_AUTHORITY};
-  UINT    x;
-  BOOL    success;
-
-  if(!OpenProcessToken(GetCurrentProcess(), TOKEN_READ, &AccessToken))
-  {
-    throw std::runtime_error(Glib::ustring::compose("Could not open process token: error code %1", (int)GetLastError()));
-  }
-
-  if(!pgwin32_get_dynamic_tokeninfo(AccessToken, TokenGroups,
-                     &InfoBuffer, errbuf, sizeof(errbuf)))
-  {
-    CloseHandle(AccessToken);
-    throw std::runtime_error(errbuf);
-  }
-
-  Groups = (PTOKEN_GROUPS) InfoBuffer;
-
-  CloseHandle(AccessToken);
-
-  if(!AllocateAndInitializeSid(&NtAuthority, 2,
-     SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0,
-                  0, &AdministratorsSid))
-  {
-    free(InfoBuffer);
-    throw std::runtime_error(Glib::ustring::compose("could not get SID for Administrators group: error code %1", (int)GetLastError()));
-  }
-
-  if(!AllocateAndInitializeSid(&NtAuthority, 2,
-  SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_POWER_USERS, 0, 0, 0, 0, 0,
-                  0, &PowerUsersSid))
-  {
-    free(InfoBuffer);
-    FreeSid(AdministratorsSid);
-    throw std::runtime_error(Glib::ustring::compose("could not get SID for PowerUsers group: error code %1", (int) GetLastError()));
-  }
-
-  success = FALSE;
-
-  for (x = 0; x < Groups->GroupCount; x++)
-  {
-    if((EqualSid(AdministratorsSid, Groups->Groups[x].Sid) && (Groups->Groups[x].Attributes & SE_GROUP_ENABLED)) ||
-      (EqualSid(PowerUsersSid, Groups->Groups[x].Sid) && (Groups->Groups[x].Attributes & SE_GROUP_ENABLED)))
-    {
-      success = TRUE;
-      break;
-    }
-  }
-
-  free(InfoBuffer);
-  FreeSid(AdministratorsSid);
-  FreeSid(PowerUsersSid);
-  return success;
-}
-
-#endif // G_OS_WIN32
-} // anonymous namespace
-
 namespace Glom
 {
 
@@ -198,7 +78,7 @@ SharedConnection::type_signal_finished SharedConnection::signal_finished()
 void SharedConnection::close()
 {
   if(m_gda_connection)
-    m_gda_connection.clear();
+    m_gda_connection.reset();
 
 
   //Tell the connection pool that we have finished with this connection.
@@ -212,10 +92,8 @@ ConnectionPool* ConnectionPool::m_instance = 0;
 
 ConnectionPool::ConnectionPool()
 :
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   m_epc_publisher(0),
   m_dialog_epc_progress(0),
-#endif // !GLOM_ENABLE_CLIENT_ONLY
   m_backend(0),
   m_sharedconnection_refcount(0),
   m_ready_to_connect(false),
@@ -291,14 +169,20 @@ sharedptr<SharedConnection> ConnectionPool::get_and_connect(std::auto_ptr<Except
   sharedptr<SharedConnection> result(0);
 
   ConnectionPool* connection_pool = ConnectionPool::get_instance();
-  if(connection_pool)
+  if(!connection_pool)
+    return result;
+
+  if(!(connection_pool->m_backend.get()))
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-    result = connection_pool->connect();
-#else
-    result = connection_pool->connect(error);
-#endif // GLIBMM_EXCEPTIONS_ENABLED
+    std::cerr << "ConnectionPool::get_and_connect(): m_backend is null." << std::endl;
+    return result; //TODO: Return a FAILURE_NO_BACKEND error?, though that would be tedious.
   }
+  
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  result = connection_pool->connect();
+#else
+  result = connection_pool->connect(error);
+#endif // GLIBMM_EXCEPTIONS_ENABLED
 
   return result;
 }
@@ -308,8 +192,10 @@ sharedptr<SharedConnection> ConnectionPool::get_and_connect(std::auto_ptr<Except
 // Store the connection for a few seconds in case it 
 // is immediately requested again, to avoid making a new connection 
 // and introspecting again, which is slow. 
+// TODO: Why aren't these member variables?
 static sharedptr<SharedConnection> connection_cached;
 static sigc::connection connection_cached_timeout_connection;
+static sigc::connection connection_cached_finished_connection;
 
 static bool on_connection_pool_cache_timeout()
 {
@@ -328,15 +214,29 @@ sharedptr<SharedConnection> ConnectionPool::connect()
 sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConnection>& error)
 #endif
 {
+  //Don't try to connect if we don't have a backend to connect to.
+  g_return_val_if_fail(m_backend.get(), sharedptr<SharedConnection>(0));
+
   if(get_ready_to_connect())
   {
+    if(connection_cached)
+    {
+      //Avoid a reconnection immediately after disconnecting:
+      return connection_cached;
+    }
     //If the connection is already open (because it is being used by somebody):
-    if(m_refGdaConnection)
+    else if(m_refGdaConnection)
     {
       sharedptr<SharedConnection> sharedConnection( new SharedConnection(m_refGdaConnection) );
 
       //Ask for notification when the SharedConnection has been finished with:
-      sharedConnection->signal_finished().connect( sigc::mem_fun(*this, &ConnectionPool::on_sharedconnection_finished) );
+      //TODO: Note that we are overwriting the connection to a signal of a
+      //previous sharedconnection here. This can be problematic when
+      //invalidate_connection() is called, because then we don't disconnect
+      //from the signal of the previous instance, and when the signal
+      //handler is called later we might decrement the reference count for
+      //a completely different shared connection.
+      connection_cached_finished_connection = sharedConnection->signal_finished().connect( sigc::mem_fun(*this, &ConnectionPool::on_sharedconnection_finished) );
 
       //Remember that somebody is using it:
       m_sharedconnection_refcount++;
@@ -349,18 +249,12 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
 
       return sharedConnection;
     }
-    else if(connection_cached)
-    {
-      //Avoid a reconnection immediately after disconnecting:
-      return connection_cached;
-    }
     else
     {
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
       std::auto_ptr<ExceptionConnection> error;
 #endif
-      if(m_backend.get())
-        m_refGdaConnection = m_backend->connect(m_database, get_user(), get_password(), error);
+      m_refGdaConnection = m_backend->connect(m_database, get_user(), get_password(), error);
 
       if(!m_refGdaConnection)
       {
@@ -373,28 +267,57 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
       {
         //Allow get_meta_store_data() to succeed:
         //Hopefully this (and the update_meta_store_for_table() calls) is all we need.
-        std::cout << "DEBUG: Calling update_meta_store_data_types() ..." << std::endl;
-        m_refGdaConnection->update_meta_store_data_types();
-        std::cout << "DEBUG: ... update_meta_store_data_types() has finished." << std::endl;
-        std::cout << "DEBUG: Calling update_meta_store_table_names() ..." << std::endl;
-        m_refGdaConnection->update_meta_store_table_names();
-        std::cout << "DEBUG: ... update_meta_store_table_names() has finished." << std::endl;
+        //std::cout << "DEBUG: Calling update_meta_store_data_types() ..." << std::endl;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+        try
+        {
+          m_refGdaConnection->update_meta_store_data_types();
+        }
+        catch(const Glib::Error& ex)
+        {
+          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex.what() << std::endl;
+        }
+        //std::cout << "DEBUG: ... update_meta_store_data_types() has finished." << std::endl;
+#else
+        std::auto_ptr<Glib::Error> ex;
+        m_refGdaConnection->update_meta_store_data_types(ex);
+        if (ex.get())
+          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex->what() << std::endl;
+#endif
+        //std::cout << "DEBUG: Calling update_meta_store_table_names() ..." << std::endl;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+        try
+        {
+          //update_meta_store_table_names() has been known to throw an exception.
+          //Glom is mostly unusable when it fails, but that's still better than a crash.
+          m_refGdaConnection->update_meta_store_table_names(m_backend->get_public_schema_name());
+        }
+        catch(const Glib::Error& ex)
+        {
+          std::cerr << "ConnectionPool::connect(): update_meta_store_table_names() failed: " << ex.what() << std::endl;
+        }
+        //std::cout << "DEBUG: ... update_meta_store_table_names() has finished." << std::endl;
+#else
+        m_refGdaConnection->update_meta_store_table_names(m_backend->get_public_schema_name(), ex);
+        if (ex.get())
+          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex->what() << std::endl;
+#endif
 
         // Connection succeeded
         // Create the fieldtypes member if it has not already been done:
         if(!m_pFieldTypes)
           m_pFieldTypes = new FieldTypes(m_refGdaConnection);
           
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifndef G_OS_WIN32
         //Let other clients discover this server via avahi:
         //TODO: Only advertize if we are the first to open the document,
         //to avoid duplicates.
         //TODO: Only advertize if it makes sense for the backend,
         //it does not for sqlite
-        avahi_start_publishing(); //Stopped in the signal_finished handler.
+        Document* document = get_document();
+        if(document && document->get_network_shared())
+          avahi_start_publishing(); //Stopped in the signal_finished handler.
 #endif // !G_OS_WIN32
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
         return connect(); //Call this method recursively. This time m_refGdaConnection exists.
@@ -412,7 +335,6 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
   return sharedptr<SharedConnection>(0);
 }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
 void ConnectionPool::create_database(const Glib::ustring& database_name)
 #else
@@ -429,7 +351,6 @@ void ConnectionPool::create_database(const Glib::ustring& database_name, std::au
   if(error.get()) throw *error;
 #endif
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void ConnectionPool::set_user(const Glib::ustring& value)
 {
@@ -443,7 +364,7 @@ void ConnectionPool::set_user(const Glib::ustring& value)
   m_user = value;
 
   //Make sure that connect() makes a new connection:
-  connection_cached.clear();
+  invalidate_connection();
 }
 
 void ConnectionPool::set_password(const Glib::ustring& value)
@@ -451,7 +372,7 @@ void ConnectionPool::set_password(const Glib::ustring& value)
   m_password = value;
   
   //Make sure that connect() makes a new connection:
-  connection_cached.clear();
+  invalidate_connection();
 }
 
 void ConnectionPool::set_database(const Glib::ustring& value)
@@ -459,7 +380,7 @@ void ConnectionPool::set_database(const Glib::ustring& value)
   m_database = value;
 
   //Make sure that connect() makes a new connection:
-  connection_cached.clear();
+  invalidate_connection();
 }
 
 Glib::ustring ConnectionPool::get_user() const
@@ -494,6 +415,15 @@ Glib::ustring ConnectionPool::get_string_find_operator() const
   return m_backend->get_string_find_operator();
 }
 
+void ConnectionPool::invalidate_connection()
+{
+  connection_cached.clear();
+  connection_cached_timeout_connection.disconnect();
+  connection_cached_finished_connection.disconnect();
+  m_refGdaConnection.reset();
+  m_sharedconnection_refcount = 0;
+}
+
 void ConnectionPool::on_sharedconnection_finished()
 {
   //g_warning("ConnectionPool::on_sharedconnection_finished().");
@@ -509,13 +439,11 @@ void ConnectionPool::on_sharedconnection_finished()
     //g_warning("ConnectionPool::on_sharedconnection_finished(): closing GdaConnection");
     m_refGdaConnection->close();
 
-    m_refGdaConnection.clear();
+    m_refGdaConnection.reset();
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifndef G_OS_WIN32
     //TODO: this should only even be started if we are the first to open the .glom file:
     avahi_stop_publishing();
-#endif
 #endif
 
     //g_warning("ConnectionPool: connection closed");
@@ -523,7 +451,7 @@ void ConnectionPool::on_sharedconnection_finished()
 }
 
 //static
-bool ConnectionPool::handle_error(bool cerr_only)
+bool ConnectionPool::handle_error_cerr_only()
 {
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   sharedptr<SharedConnection> sharedconnection = get_and_connect();
@@ -555,25 +483,6 @@ bool ConnectionPool::handle_error(bool cerr_only)
         }
       }
 
-      //For debugging only:
-      //Gtk::Dialog* dialog = 0;
-      //dialog->run(); //Force a crash.
-
-      if(!cerr_only)
-      {
-#ifdef GLOM_ENABLE_MAEMO
-        Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, error_details);
-#else
-        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Internal error")), true, Gtk::MESSAGE_WARNING );
-        dialog.set_secondary_text(error_details);
-#endif
-        //TODO: dialog.set_transient_for(*get_application());
-        dialog.run(); //TODO: This segfaults in gtk_window_set_modal() when this method is run a second time, for instance if there are two database errors.
-#ifdef GLOM_CONNECTION_DEBUG
-        std::cout << "debug: after Internal Error dialog run()." << std::endl;
-#endif
-      }
-
       return true; //There really was an error.
     }
   }
@@ -600,9 +509,8 @@ static void on_linux_signal(int signum)
 
   if(signum == SIGSEGV)
   {
-    //TODO: Make this dialog transient for the parent window, 
-    //though this is obviously an unusual case.
-    connection_pool->cleanup(0 /* parent_window */);
+    ConnectionPool::SlotProgress slot_ignored;
+    connection_pool->cleanup(slot_ignored);
 
     //Let GNOME/Ubuntu's crash handler still handle this?
     if(previous_sig_handler)
@@ -612,12 +520,12 @@ static void on_linux_signal(int signum)
   }
 }
 
-bool ConnectionPool::startup(Gtk::Window* parent_window)
+bool ConnectionPool::startup(const SlotProgress& slot_progress, bool network_shared)
 {
   if(!m_backend.get())
     return false;
 
-  if(!m_backend->startup(parent_window))
+  if(!m_backend->startup(slot_progress, network_shared))
     return false;
 
 #ifndef G_OS_WIN32
@@ -632,13 +540,23 @@ bool ConnectionPool::startup(Gtk::Window* parent_window)
   return true;
 }
 
-void ConnectionPool::cleanup(Gtk::Window* parent_window)
+void ConnectionPool::cleanup(const SlotProgress& slot_progress)
 {
+  // Without a valid backend instance we should not state that we are ready to
+  // connect. Fixes crash described in #577821.
+
+  // TODO: Implement checks in set_ready_to_connect() to only set the flag to
+  // "true" when the ConnectionPool instance is in a consistent state
+  // afterwards. Or, have a private set_ready_to_connect() and a public
+  // set_not_ready_to_connect()?
+  set_ready_to_connect(false);
+
   if(m_backend.get())
-    m_backend->cleanup(parent_window);
+    m_backend->cleanup(slot_progress);
+
+  //Make sure that connect() tries to make a new connection:
+  invalidate_connection();
 
-  //Make sure that connect() makes a new connection:
-  connection_cached.clear();
 
 #ifndef G_OS_WIN32
   /* Stop advertising the self-hosting database server via avahi: */
@@ -650,7 +568,15 @@ void ConnectionPool::cleanup(Gtk::Window* parent_window)
   previous_sig_handler = SIG_DFL; /* Arbitrary default */
 }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
+
+bool ConnectionPool::set_network_shared(const SlotProgress& slot_progress, bool network_shared)
+{
+  if(m_backend.get())
+    return m_backend->set_network_shared(slot_progress, network_shared);
+  else
+    return false;
+}
+
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
 bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field)
 #else
@@ -672,19 +598,20 @@ bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr
 
   if(!m_refGdaConnection)
     return false;
-
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   std::auto_ptr<Glib::Error> error;
-  const bool result = m_backend->add_column(m_refGdaConnection, table_name, field, error);
-  if(error.get()) throw *error;
-
-  m_refGdaConnection->update_meta_store_table(table_name);
-#else
-  const bool result = m_backend->add_column(m_refGdaConnection, table_name, field, error);
-  if(result)
-    result = m_refGdaConnection->update_meta_store_table(table_name, error);
 #endif
-
+  const bool result = m_backend->add_column(m_refGdaConnection, table_name, field, error);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  if(error.get()) throw *error;
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
+#else
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl; 
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl;   
+#endif
   return result;
 }
 
@@ -712,16 +639,18 @@ bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::us
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   std::auto_ptr<Glib::Error> error;
-  const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name, error);
-  if(error.get()) throw *error;
-
-  m_refGdaConnection->update_meta_store_table(table_name);
-#else
-  const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name, error);
-  if(result)
-    result = m_refGdaConnection->update_meta_store_table(table_name, error);
 #endif
-
+  const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name, error);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  if(error.get()) throw *error;
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
+#else
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl; 
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl; 
+#endif
   return result;
 }
 
@@ -731,8 +660,8 @@ bool ConnectionPool::change_column(const Glib::ustring& table_name, const shared
 bool ConnectionPool::change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field, std::auto_ptr<Glib::Error>& error)
 #endif
 {
-  type_vecConstFields old_fields(1, field_old);
-  type_vecConstFields new_fields(1, field);
+  type_vec_const_fields old_fields(1, field_old);
+  type_vec_const_fields new_fields(1, field);
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   return change_columns(table_name, old_fields, new_fields);
@@ -742,9 +671,9 @@ bool ConnectionPool::change_column(const Glib::ustring& table_name, const shared
 }
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vecConstFields& old_fields, const type_vecConstFields& new_fields)
+bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields)
 #else
-bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vecConstFields& old_fields, const type_vecConstFields& new_fields, std::auto_ptr<Glib::Error>& error)
+bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields, std::auto_ptr<Glib::Error>& error)
 #endif
 {
   sharedptr<SharedConnection> conn;
@@ -765,74 +694,29 @@ bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   std::auto_ptr<Glib::Error> error;
-  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields, error);
-  if(error.get()) throw *error;
-
-  m_refGdaConnection->update_meta_store_table(table_name);
-#else
-  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields, error);
-  if(result)
-    result = m_refGdaConnection->update_meta_store_table(table_name, error);
 #endif
+  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields, error);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  if(error.get()) throw *error;
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
 
+#else
+  if(error.get())
+    std::cerr << "Error: " << error->what() << std::endl; 
+  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
+#endif
   return result;
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-bool ConnectionPool::initialize(Gtk::Window* parent_window)
+ConnectionPool::InitErrors ConnectionPool::initialize(const SlotProgress& slot_progress, bool network_shared)
 {
   if(m_backend.get())
-    return m_backend->initialize(parent_window, get_user(), get_password());
+    return m_backend->initialize(slot_progress, get_user(), get_password(), network_shared);
   else
-    return false;
+    return Backend::INITERROR_OTHER;
 }
 
-bool ConnectionPool::check_user_is_not_root()
-{
-  Glib::ustring message;
-#ifdef G_OS_WIN32
-  try
-  {
-    if(pgwin32_is_admin())
-    {
-      message = _("You seem to be running Glom as a user with administrator privileges. Glom may not be run with such privileges for security reasons.\nPlease login to your system as a normal user.");
-    }
-  }
-  catch(const std::runtime_error& ex)
-  {
-    message = ex.what();
-  }
-#else
-  //std::cout << "ConnectionPool::check_user_is_not_root(): geteuid()=" << geteuid() << ", getgid()=" << getgid() << std::endl;
-
-  //This is very linux-specific. We should ifdef this out for other platforms.
-  if(geteuid() == 0)
-  {
-    //Warn the user:
-    message = _("You seem to be running Glom as root. Glom may not be run as root.\nPlease login to your system as a normal user.");
-  }
-#endif
-
-  if(!message.empty())
-  {
-#ifndef GLOM_ENABLE_MAEMO
-    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Running As Root")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
-    dialog.set_secondary_text(message);
-    dialog.run();
-#else
-    Hildon::Note note(Hildon::NOTE_TYPE_INFORMATION, message);
-    note.run();
-#endif
-
-    return false; /* Is root. Bad. */
-  }
-
-  return true; /* Not root. It's OK. */
-}
-
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-Document_Glom* ConnectionPool::get_document()
+Document* ConnectionPool::get_document()
 {
   if(!m_slot_get_document)
   {
@@ -842,9 +726,7 @@ Document_Glom* ConnectionPool::get_document()
 
   return m_slot_get_document();
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifndef G_OS_WIN32
 //static
 EpcContents* ConnectionPool::on_publisher_document_requested(EpcPublisher* /* publisher */, const gchar* /* key */, gpointer user_data)
@@ -853,7 +735,7 @@ EpcContents* ConnectionPool::on_publisher_document_requested(EpcPublisher* /* pu
   if(!connection_pool)
     return 0;
 
-  const Document_Glom* document = connection_pool->get_document();
+  const Document* document = connection_pool->get_document();
   if(!document)
     return 0;
 
@@ -901,45 +783,34 @@ gboolean ConnectionPool::on_publisher_document_authentication(EpcAuthContext* co
 void ConnectionPool::on_epc_progress_begin(const gchar* /* title */, gpointer user_data)
 {
   //We ignore the title parameter because there is no way that libepc could know what Glom wants to say.
- 
+  
   ConnectionPool* connection_pool = (ConnectionPool*)user_data;
-
-  //Create the dialog:
-  if(connection_pool->m_dialog_epc_progress)
-  {
-    delete connection_pool->m_dialog_epc_progress;
-    connection_pool->m_dialog_epc_progress = 0;
-  }
-
-  Gtk::MessageDialog* message_dialog = new Gtk::MessageDialog(Bakery::App_Gtk::util_bold_message(_("Glom: Generating Encryption Certificates")), true, Gtk::MESSAGE_INFO);
-  message_dialog->set_secondary_text(_("Please wait while Glom prepares your system for publishing over the network."));
-  message_dialog->show();
-
-  connection_pool->m_dialog_epc_progress = message_dialog; 
+  if(connection_pool)
+    connection_pool->m_epc_slot_begin();
 }
 
-void ConnectionPool::on_epc_progress_update(gdouble /* progress */, const gchar* /* message */, gpointer /* user_data */)
+void ConnectionPool::on_epc_progress_update(gdouble /* progress */, const gchar* /* message */, gpointer user_data)
 {
   //We ignore the title parameter because there is no way that libepc could know what Glom wants to say.
   //TODO: Show the progress in a ProgressBar.
 
-  //ConnectionPool* connection_pool = (ConnectionPool*)user_data;
-
-  //Allow GTK+ to process events, so that the UI is responsive:
-  while(Gtk::Main::events_pending())
-   Gtk::Main::iteration();
+  ConnectionPool* connection_pool = (ConnectionPool*)user_data;
+  if(connection_pool)
+    connection_pool->m_epc_slot_progress();
 }
 
 void ConnectionPool::on_epc_progress_end(gpointer user_data)
 {
   ConnectionPool* connection_pool = (ConnectionPool*)user_data;
+  if(connection_pool)
+    connection_pool->m_epc_slot_done();
+}
 
-  //Delete the dialog:
-  if(connection_pool->m_dialog_epc_progress)
-  {
-    delete connection_pool->m_dialog_epc_progress;
-    connection_pool->m_dialog_epc_progress = 0;
-  }
+void ConnectionPool::set_avahi_publish_callbacks(const type_void_slot& slot_begin, const type_void_slot& slot_progress, const type_void_slot& slot_done)
+{
+  m_epc_slot_begin = slot_begin;
+  m_epc_slot_progress = slot_progress;
+  m_epc_slot_done = slot_done; 
 }
 
 
@@ -958,26 +829,26 @@ void ConnectionPool::avahi_start_publishing()
 #endif
   
   //Publish the document contents over HTTPS (discoverable via avahi):
-  const Document_Glom* document = get_document();
+  const Document* document = get_document();
   if(!document)
     return;
 
-  m_epc_publisher = epc_publisher_new(document->get_database_title().c_str(), "glom", NULL);
+  m_epc_publisher = epc_publisher_new(document->get_database_title().c_str(), "glom", 0);
   epc_publisher_set_protocol(m_epc_publisher, publish_protocol);
   
-  epc_publisher_add_handler(m_epc_publisher, "document", on_publisher_document_requested, this /* user_data */, NULL);
+  epc_publisher_add_handler(m_epc_publisher, "document", on_publisher_document_requested, this /* user_data */, 0);
 
   //Password-protect the document,
   //because the document's structure could be considered as a business secret:
   epc_publisher_set_auth_flags(m_epc_publisher, EPC_AUTH_PASSWORD_TEXT_NEEDED);
-  epc_publisher_set_auth_handler(m_epc_publisher, "document", on_publisher_document_authentication, this /* user_data */, NULL);
+  epc_publisher_set_auth_handler(m_epc_publisher, "document", on_publisher_document_authentication, this /* user_data */, 0);
 
   //Install progress callback, so we can keep the UI responsive while libepc is generating certificates for the first time:
   EpcShellProgressHooks callbacks;
   callbacks.begin = &ConnectionPool::on_epc_progress_begin;
   callbacks.update = &ConnectionPool::on_epc_progress_update;
   callbacks.end = &ConnectionPool::on_epc_progress_end;
-  epc_shell_set_progress_hooks(&callbacks, this, NULL);
+  epc_shell_set_progress_hooks(&callbacks, this, 0);
       
   //Prevent the consumer from seeing duplicates,
   //if multiple client computers advertize the same document:
@@ -1016,7 +887,7 @@ void ConnectionPool::avahi_stop_publishing()
   epc_publisher_quit(m_epc_publisher);
 #endif // !G_OS_WIN32
   g_object_unref(m_epc_publisher);
-  m_epc_publisher = NULL;
+  m_epc_publisher = 0;
 }
 #endif // !G_OS_WIN32
 
@@ -1025,7 +896,5 @@ void ConnectionPool::set_get_document_func(const SlotGetDocument& slot)
   m_slot_get_document = slot;
 }
 
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
 
 } //namespace Glom
