@@ -24,6 +24,7 @@
 #include <glibmm/thread.h>
 #include <glibmm/iochannel.h>
 #include <glibmm/shell.h>
+#include <glibmm/fileutils.h>
 #include <glibmm/miscutils.h>
 #include <glibmm/i18n.h>
 #include <memory> //For auto_ptr.
@@ -106,10 +107,10 @@ private:
     // it can write to it.
     SECURITY_ATTRIBUTES security_attr;
     security_attr.nLength = sizeof(security_attr);
-    security_attr.lpSecurityDescriptor = NULL;
+    security_attr.lpSecurityDescriptor = 0;
     security_attr.bInheritHandle = TRUE;
 
-    HANDLE result = CreateFile(filename.c_str(), GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_DELETE, &security_attr, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+    HANDLE result = CreateFile(filename.c_str(), GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_DELETE, &security_attr, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
     if(!result) throw SpawnError(win32_error_message());
 #endif // GLIBMM_EXCEPTIONS_ENABLED
@@ -184,17 +185,19 @@ private:
 public:
   typedef sigc::signal<void> SignalFinished;
 
+  /** TODO: Document the redirect parameter.
+   */
   SpawnInfo(const Glib::ustring& command_line, int redirect):
     running(false), return_status(0)
   {
 #ifdef G_OS_WIN32
     startup_info.cb = sizeof(startup_info);
     startup_info.dwFlags = STARTF_USESTDHANDLES;
-    startup_info.lpReserved = NULL;
-    startup_info.lpDesktop = NULL;
-    startup_info.lpTitle = NULL;
+    startup_info.lpReserved = 0;
+    startup_info.lpDesktop = 0;
+    startup_info.lpTitle = 0;
     startup_info.cbReserved2 = 0;
-    startup_info.lpReserved2 = NULL;
+    startup_info.lpReserved2 = 0;
     startup_info.hStdInput = INVALID_HANDLE_VALUE;
 
     if(redirect & REDIRECT_STDOUT)
@@ -212,7 +215,7 @@ public:
     std::copy(command_line.data(), command_line.data() + command_line.length(), command.begin());
     command[command_line.length()] = '\0';
 
-    if(!CreateProcess(NULL, &command[0], NULL, NULL, TRUE, CREATE_NO_WINDOW, NULL, NULL, &startup_info, &process_info))
+    if(!CreateProcess(0, &command[0], 0, 0, TRUE, CREATE_NO_WINDOW, 0, 0, &startup_info, &process_info))
     {
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
       throw SpawnError(win32_error_message());
@@ -227,10 +230,10 @@ public:
       int child_stdout;
       int child_stderr;
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
-      Glib::spawn_async_with_pipes(Glib::get_current_dir(), arguments, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid, NULL, redirect & REDIRECT_STDOUT ? &child_stdout : NULL, redirect & REDIRECT_STDERR ? &child_stderr : NULL);
+      Glib::spawn_async_with_pipes(Glib::get_current_dir(), arguments, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid, 0, redirect & REDIRECT_STDOUT ? &child_stdout : 0, redirect & REDIRECT_STDERR ? &child_stderr : 0);
 #else
       std::auto_ptr<Glib::Error> error;
-      Glib::spawn_async_with_pipes(Glib::get_current_dir(), arguments, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid, NULL, redirect & REDIRECT_STDOUT ? &child_stdout : NULL, redirect & REDIRECT_STDERR ? &child_stderr : NULL, error);
+      Glib::spawn_async_with_pipes(Glib::get_current_dir(), arguments, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid, 0, redirect & REDIRECT_STDOUT ? &child_stdout : 0, redirect & REDIRECT_STDERR ? &child_stderr : 0, error);
       if (error.get())
         std::cerr << "Spawn Error: " << error->what() << std::endl;
 #endif
@@ -335,12 +338,16 @@ private:
 #endif
 };
 
-std::auto_ptr<const SpawnInfo> spawn_async(const Glib::ustring& command_line, int redirect)
+static std::auto_ptr<const SpawnInfo> spawn_async(const Glib::ustring& command_line, int redirect)
 {
   return std::auto_ptr<const SpawnInfo>(new SpawnInfo(command_line, redirect));
 }
 
-bool spawn_async_end(std::auto_ptr<const SpawnInfo> info, std::string* stdout_text = NULL, std::string* stderr_text = NULL, int* return_status = NULL)
+/**
+ * @param return_status: The return value of the command.
+ * @result Whether we successfully ended the async spawn. 
+ */
+static bool spawn_async_end(std::auto_ptr<const SpawnInfo> info, std::string* stdout_text = 0, std::string* stderr_text = 0, int* return_status = 0)
 {
   if(stdout_text)
     info->get_stdout(*stdout_text);
@@ -354,7 +361,7 @@ bool spawn_async_end(std::auto_ptr<const SpawnInfo> info, std::string* stdout_te
   return !info->is_running();
 }
 
-int spawn_sync(const Glib::ustring& command_line, std::string* stdout_text, std::string* stderr_text)
+static int spawn_sync(const Glib::ustring& command_line, std::string* stdout_text, std::string* stderr_text)
 {
   int redirect_flags = 0;
   if(stdout_text)
@@ -373,14 +380,13 @@ int spawn_sync(const Glib::ustring& command_line, std::string* stdout_text, std:
   mainloop->run();
 
   int return_status = 0;
-  bool returned = spawn_async_end(info, stdout_text, stderr_text, &return_status);
+  const bool returned = spawn_async_end(info, stdout_text, stderr_text, &return_status);
   g_assert(returned);
   return return_status;
 }
 
 } // namespace Impl
 
-
 bool execute_command_line_and_wait(const std::string& command, const SlotProgress& slot_progress)
 {
   //Show UI progress feedback while we wait for the command to finish:
@@ -404,11 +410,58 @@ bool execute_command_line_and_wait(const std::string& command, const SlotProgres
   timeout_connection.disconnect();
   
   int return_status = false;
-  const bool returned = Impl::spawn_async_end(info, NULL, NULL, &return_status);
+  const bool returned = Impl::spawn_async_end(info, 0, 0, &return_status);
   if(!returned)
     return false; // User closed the dialog prematurely?
 
-  return (return_status == 0);
+  return (return_status == EXIT_SUCCESS);
+}
+
+bool execute_command_line_and_wait(const std::string& command, const SlotProgress& slot_progress, std::string& output)
+{
+  //Initialize output parameter:
+  output = std::string();
+
+  //Show UI progress feedback while we wait for the command to finish:
+  
+  std::auto_ptr<const Impl::SpawnInfo> info = Impl::spawn_async(command, Impl::REDIRECT_STDOUT | Impl::REDIRECT_STDERR);
+  
+  Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
+  info->signal_finished().connect(
+    sigc::bind(sigc::ptr_fun(&on_spawn_info_finished), sigc::ref(mainloop) ) );
+
+  // Pulse two times a second:
+  sigc::connection timeout_connection = Glib::signal_timeout().connect(
+    sigc::bind_return( slot_progress, true),
+    500);
+  slot_progress(); //Make sure it is called at least once.
+
+  //Block until signal_finished is called.
+  mainloop->run();
+
+  //Stop the timeout callback:
+  timeout_connection.disconnect();
+  
+  int return_status = false;
+  std::string stdout_text, stderr_text;
+  const bool returned = Impl::spawn_async_end(info, &stdout_text, &stderr_text, &return_status);
+  if(!returned)
+    return false; // User closed the dialog prematurely?
+
+  //std::cout << "DEBUG: command=" << command << std::endl;
+  //std::cout << "  DEBUG: stdout_text=" << stdout_text << std::endl;
+  //std::cout << "  DEBUG: stderr_text=" << stderr_text << std::endl;
+
+  output = stdout_text;
+
+  if(!stderr_text.empty())
+  {
+    std::cerr << "Glom: execute_command_line_and_wait(): command produced stderr text: " << std::endl <<
+      "  command: " << command << std::endl << 
+      "  error text: " << stderr_text << std::endl;
+  }
+
+  return (return_status == EXIT_SUCCESS);
 }
 
 // Callback handlers for execute_command_line_and_wait_until_second_command_returns_success
@@ -447,7 +500,7 @@ namespace
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
-      return_status = Impl::spawn_sync(second_command, &stdout_output, NULL);
+      return_status = Impl::spawn_sync(second_command, &stdout_output, 0);
     }
     catch(const Impl::SpawnError& ex)
     {
@@ -456,7 +509,7 @@ namespace
       // a row or so.
     }
 #else
-    Impl::spawn_sync(second_command, &stdout_output, NULL);
+    return_status = Impl::spawn_sync(second_command, &stdout_output, 0);
 #endif
 
     if(!success_text.empty())
@@ -470,7 +523,7 @@ namespace
       Glib::setenv("LANGUAGE", stored_env_language, true /* overwrite */);
     }
 
-    if(return_status == 0)
+    if(return_status == EXIT_SUCCESS)
     {
       bool success = true; //Just check the return code.
       if(!success_text.empty()) //Check the output too.
@@ -509,6 +562,7 @@ namespace
 
 } //Anonymous namespace
 
+/*
 static bool on_timeout_delay(const Glib::RefPtr<Glib::MainLoop>& mainloop)
 {
   //Allow our mainloop.run() to return:
@@ -517,6 +571,7 @@ static bool on_timeout_delay(const Glib::RefPtr<Glib::MainLoop>& mainloop)
     
   return false;
 }
+*/
 
 bool execute_command_line_and_wait_until_second_command_returns_success(const std::string& command, const std::string& second_command, const SlotProgress& slot_progress, const std::string& success_text)
 {
@@ -547,11 +602,15 @@ bool execute_command_line_and_wait_until_second_command_returns_success(const st
   watch_conn.disconnect();
 
   std::string stderr_text;
-
-  const bool success = Impl::spawn_async_end(info, NULL, &stderr_text, NULL);
-
-  if(success) //response == Gtk::RESPONSE_OK)
+  int return_status = 0;
+  const bool success = Impl::spawn_async_end(info, 0, &stderr_text, &return_status);
+  
+  if(success && (return_status == EXIT_SUCCESS))
   {
+    /* Don't sleep here. Instead we just keep trying to connect until it succeeds, 
+     * timing out during that if necessary.
+     *
+     *
     //Sleep for a bit more, because I think that pg_ctl sometimes reports success too early.
     Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
     sigc::connection connection_timeout = Glib::signal_timeout().connect(
@@ -560,7 +619,8 @@ bool execute_command_line_and_wait_until_second_command_returns_success(const st
     mainloop->run();
 
     connection_timeout.disconnect();
-    
+    */
+
     return true;
   }
   else
