@@ -23,6 +23,9 @@
 
 #include <gtkmm/treeviewcolumn.h>
 
+namespace Glom
+{
+
 class DbTreeViewColumnGlom : public Gtk::TreeViewColumn
 {
 public:
@@ -32,8 +35,10 @@ public:
   virtual Glib::ustring get_column_id() const;
   virtual void set_column_id(const Glib::ustring& value);
 
-protected:
+private:
   Glib::ustring m_column_id; 
 };
 
+} //namespace Glom
+
 #endif //GLOM_DB_ADDDEL_TREEVIEWCOLUMN_GLOM_H
