@@ -44,9 +44,21 @@ public:
 
   void set_columns_count(guint value);
 
-  /** Sets the padding to put between the child widgets.
+  /** Sets the padding to put between the columns of widgets.
    */
-  virtual void set_padding(guint padding);
+  void set_column_padding(guint padding);
+  
+  /** Gets the padding between the columns of widgets.
+   */
+  guint get_column_padding() const;
+  
+  /** Sets the padding to put between the rows of widgets.
+   */
+  void set_row_padding(guint padding);
+  
+  /** Gets the padding between the rows of widgets.
+   */
+  guint get_row_padding() const;
 
   /** Show extra UI that is useful in RAD tools:
    */
@@ -58,9 +70,9 @@ public:
   typedef sigc::slot<void, Widget&> ForallSlot;
   void forall(const ForallSlot& slot);
 
-protected:
+private:
 
-#ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+  #ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   // These are the hand-coded C default signal handlers in case the
   // corresponding glibmm API has been disabled
   static void glom_size_request_impl(GtkWidget* widget, GtkRequisition* requisition);
@@ -71,7 +83,7 @@ protected:
   static void glom_realize_impl(GtkWidget* widget);
   static void glom_unrealize_impl(GtkWidget* widget);
   static gboolean glom_expose_event_impl(GtkWidget* widget, GdkEventExpose* event);
-#endif
+  #endif
   //Overrides:
 
   //Handle child widgets:
@@ -83,10 +95,12 @@ protected:
   virtual void on_remove(Gtk::Widget* child);
 
   //Do extra drawing:
-  virtual void on_realize();
-  virtual void on_unrealize();
-  virtual bool on_expose_event(GdkEventExpose* event);
+  //Virtual method overrides:
+  void on_realize();
+  void on_unrealize();
+  bool on_expose_event(GdkEventExpose* event);
 
+protected:
   int get_column_height(guint start_widget, guint widget_count, int& total_width) const;
 
   /** 
@@ -115,7 +129,7 @@ protected:
     Gtk::Allocation m_second_allocation;
   };
 
-  typedef std::vector<FlowTableItem> type_vecChildren;
+private:
   void insert_before(FlowTableItem& item, Gtk::Widget& before);
 
   int get_item_requested_height(const FlowTableItem& item) const;
@@ -127,9 +141,12 @@ protected:
   Gtk::Allocation assign_child(Gtk::Widget* widget, int x, int y);
   Gtk::Allocation assign_child(Gtk::Widget* widget, int x, int y, int width, int height);
 
+protected:
+  typedef std::vector<FlowTableItem> type_vecChildren;
   type_vecChildren m_children;
+private:
   guint m_columns_count;
-  guint m_padding;
+  guint m_column_padding, m_row_padding;
   bool m_design_mode;
 
   //Lines to draw in on_expose_event:
