@@ -26,6 +26,7 @@
 #include <gtkmm/button.h>
 
 #ifdef GLOM_ENABLE_MAEMO
+#include <gtkmm/alignment.h>
 #include <hildonmm/stackable-window.h>
 #endif
 
@@ -47,6 +48,10 @@ public:
 
 private:
 
+  #ifdef GLOM_ENABLE_MAEMO
+  Gtk::Alignment m_alignment;
+  #endif
+
   //Signal handlers:
   void on_box_cancelled();
 };
